@@ -1,11 +1,11 @@
 // Re-export the io::Result / Error types for convenience
+// pub use std::io::{Error, ErrorKind, Read, Result, Write};
 pub use std::io::{ErrorKind, Result};
 
 // TODO: Delete this
 /// A helper trait to provide the map_non_block function on Results.
 pub trait MapNonBlock<T> {
-    /// Maps a `Result<T>` to a `Result<Option<T>>` by converting
-    /// operation-would-block errors into `Ok(None)`.
+    #[allow(dead_code)]
     fn map_non_block(self) -> Result<Option<T>>;
 }
 
