@@ -91,7 +91,7 @@ pub struct CachedRunData {
     pub coords: Vec<i16>,
     pub span: FragmentStyle,
     pub line: u32,
-    pub hash: Option<u64>,
+    pub hash: u64,
     pub font: usize,
     pub size: f32,
     pub ascent: f32,
@@ -178,7 +178,7 @@ impl RenderData {
         font: &usize,
         size: f32,
         line: u32,
-        hash: Option<u64>,
+        hash: u64,
         shaper: Shaper<'_>,
     ) {
         // In case is a new line,
@@ -504,28 +504,6 @@ impl<'a> Run<'a> {
         self.run.span.decoration_color
     }
 
-    /// Returns the underline offset for the run.
-    // #[inline]
-    // pub fn underline_offset(&self) -> f32 {
-    //     // span_data.underline_offset.unwrap_or(metrics.underline_offset),
-    //     self.run.span.underline_offset.unwrap_or(0.0)
-    // }
-
-    // /// Returns the underline color for the run.
-    // #[inline]
-    // pub fn underline_color(&self) -> [f32; 4] {
-    //     self.run.span.underline_color.unwrap_or(self.run.span.color)
-    // }
-
-    // /// Returns the underline size for the run.
-    // #[inline]
-    // pub fn underline_size(&self) -> f32 {
-    //     self.run
-    //         .span
-    //         .underline_size
-    //         .unwrap_or(self.run.strikeout_size)
-    // }
-
     /// Returns an iterator over the clusters in logical order.
     #[inline]
     pub fn clusters(&self) -> Clusters<'a> {
@@ -828,8 +806,8 @@ impl<'a> Line<'a> {
     }
 
     #[inline]
-    pub fn hash(&self) -> &Option<u64> {
-        &self.line.hash
+    pub fn hash(&self) -> u64 {
+        self.line.hash
     }
 }
 
