@@ -5,8 +5,9 @@
 
 use crate::components::text::glyph::OwnedSection;
 use crate::sugarloaf::graphics;
-use crate::sugarloaf::tree::SugarTree;
-use crate::sugarloaf::{PxScale, Rect, SugarText};
+use crate::sugarloaf::state::SugarTree;
+use crate::sugarloaf::{PxScale, Rect};
+use crate::Text;
 
 #[allow(unused)]
 struct GraphicRect {
@@ -23,7 +24,6 @@ struct GraphicRect {
 #[derive(Default)]
 pub struct Elementary {
     pub rects: Vec<Rect>,
-    pub should_resize: bool,
     text_y: f32,
     current_row: u16,
 }
@@ -34,23 +34,17 @@ impl Elementary {
         &self.rects
     }
 
-    #[inline]
-    pub fn set_should_resize(&mut self) {
-        self.should_resize = true;
-    }
-
     #[inline]
     pub fn clean(&mut self) {
         self.current_row = 0;
         self.text_y = 0.0;
-        self.should_resize = false;
         self.rects.clear();
     }
 
     #[inline]
     pub fn create_section_from_text(
         &mut self,
-        sugar_text: &SugarText,
+        sugar_text: &Text,
         tree: &SugarTree,
     ) -> OwnedSection {
         let text = crate::components::text::OwnedText {
